const express = require('express');

const outOfSchoolApp = express.Router();

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const id2str = require(`${libs}/middlewares/id2str`);

const addMissing = require(`${libs}/middlewares/addMissing`);

const config = require(`${libs}/config`);

const cache = require('apicache').options({ debug: config.debug, statusCodes: {include: [200]} }).middleware;

let rqf = new ReqQueryFields();

outOfSchoolApp.use(cache('15 day'));

outOfSchoolApp.get('/year_range', (req, res, next) => {
    req.sql.from('pnad')
    .field('MIN(pnad.ano_censo)', 'start_year')
    .field('MAX(pnad.ano_censo)', 'end_year');
    next();
}, query, response('range'));

outOfSchoolApp.get('/years', (req, res, next) => {
    req.sql.from('pnad').
    field('DISTINCT pnad.ano_censo', 'year');
    next();
}, query, response('years'));

outOfSchoolApp.get('/full_age_range', (req, res, next) => {
    req.result = [
        {id: 1, name: '0-3'},
        {id: 2, name: '4-5'},
        {id: 3, name: '6-10'},
        {id: 4, name: '11-14'},
        {id: 5, name: '15-17'},
        {id: 6, name: '18-24'},
        {id: 7, name: '25-29'},
        {id: 8, name: '30-40'},
        {id: 9, name: '41-50'},
        {id: 10, name: '51-64'},
        {id: 11, name: '64+'}
    ];
    next();
}, response('full_age_range'));

outOfSchoolApp.get('/ethnic_group_pnad', (req, res, next) => {
    req.result = [
        {id: 0, name: 'Indígena'},
        {id: 1, name: 'Branca e amarela'},
        {id: 2, name: 'Preta e parda'},
        {id: 9, name: 'Sem declaração'}
    ];
    next();
}, response('ethnic_group_pnad'));

outOfSchoolApp.get('/location', (req, res, next) => {
    req.result = [
        {id: 1, name: 'Urbana'},
        {id: 2, name: 'Rural'}
    ];
    next();
}, response('location'));

outOfSchoolApp.get('/gender', (req, res, next) => {
    req.result = [
        {id: 1, name: 'Masculino'},
        {id: 2, name: 'Feminino'}
    ];
    next();
}, response('gender'));

outOfSchoolApp.get('/fifth_household_income', (req, res, next) => {
    req.result = [
		{id: 1, name: '20% menores'},
        {id: 2, name: '2o quinto'},
        {id: 3, name: '3o quinto'},
        {id: 4, name: '4o quinto'},
		{id: 5, name: '20% maiores'},
        {id: -1, name: 'Sem declaração'}
	];
    next();
},response('fifth_household_income'));

outOfSchoolApp.get('/extremes_household_income', (req, res, next) => {
    req.result = [
		{id: 1, name: '10% menores'},
        {id: 2, name: '10% maiores'},
        {id: -1, name: 'Sem declaração'}
	];
    next();
}, response('extremes_household_income'));

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'region',
    table: 'regiao',
    tableField: 'nome',
    resultField: 'region_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'regiao_id',
        foreignTable: 'pnad'
    }
}).addValue({
    name: 'state',
    table: 'estado',
    tableField: 'nome',
    resultField: 'state_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'estado_id',
        foreignTable: 'pnad'
    }
}).addValue({
    name: 'ethnic_group_pnad',
    table: 'pnad',
    tableField: 'cor_raca',
    resultField: 'ethnic_group_pnad_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'cor_raca'
    }
}).addValue({
    name: 'min_year',
    table: 'pnad',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        table: 'pnad',
        field: 'ano_censo'
    }
}).addValue({
    name: 'max_year',
    table: 'pnad',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        table: 'pnad',
        field: 'ano_censo'
    }
}).addValue({
    name: 'full_age_range',
    table: 'pnad',
    tableField: 'idade_31_03',
    resultField: 'full_age_range_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'faixa_etaria_31_03'
    }
}).addValue({
    name: 'gender',
    table: 'pnad',
    tableField: 'sexo',
    resultField: 'gender_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'sexo'
    }
}).addValue({
    name: 'location',
    table: 'pnad',
    tableField: 'localizacao_id',
    resultField: 'location_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'localizacao_id'
    }
}).addValue({
    name: 'extremes_household_income',
    table: 'pnad',
    tableField: 'extremos_nivel_rendimento',
    resultField: 'extremes_household_income_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'extremos_nivel_rendimento'
    }
}).addValue({
    name: 'fifth_household_income',
    table: 'pnad',
    tableField: 'quintil_nivel_rendimento',
    resultField: 'fifth_household_income_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'quintil_nivel_rendimento'
    }
});

outOfSchoolApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.from('pnad')
    .field('SUM(pnad.peso)', 'total')
    .field('pnad.ano_censo', 'year')
    .where('pnad.escolaridade_familiar >= 1 AND pnad.escolaridade_familiar <= 4 AND pnad.frequenta_escola_creche = 4')
    .group('pnad.ano_censo')
    .order('pnad.ano_censo');

    next();
}, query, addMissing(rqf), id2str.transform(), response('out_of_school'));

module.exports = outOfSchoolApp;